@@ -1,7 +1,6 @@
 """
 Algebraic connectivity and Fiedler vectors of undirected graphs.
 """
-from functools import partial
 
 import networkx as nx
 from networkx.utils import (
@@ -51,7 +50,6 @@ class _PCGSolver:
     def _solve(self, b, tol):
         import numpy as np
         import scipy as sp
-        import scipy.linalg.blas  # call as sp.linalg.blas
 
         A = self._A
         M = self._M
@@ -89,7 +87,6 @@ class _LUSolver:
 
     def __init__(self, A):
         import scipy as sp
-        import scipy.sparse.linalg  # call as sp.sparse.linalg
 
         self._LU = sp.sparse.linalg.splu(
             A,
@@ -185,9 +182,6 @@ def _tracemin_fiedler(L, X, normalized, tol, method):
     """
     import numpy as np
     import scipy as sp
-    import scipy.linalg  # call as sp.linalg
-    import scipy.linalg.blas  # call as sp.linalg.blas
-    import scipy.sparse  # call as sp.sparse
 
     n = X.shape[0]
 
@@ -195,8 +189,7 @@ def _tracemin_fiedler(L, X, normalized, tol, method):
         # Form the normalized Laplacian matrix and determine the eigenvector of
         # its nullspace.
         e = np.sqrt(L.diagonal())
-        # TODO: rm csr_array wrapper when spdiags array creation becomes available
-        D = sp.sparse.csr_array(sp.sparse.spdiags(1 / e, 0, n, n, format="csr"))
+        D = sp.sparse.dia_array((1 / e, 0), shape=(n, n)).tocsr()
         L = D @ L @ D
         e *= 1.0 / np.linalg.norm(e, 2)
 
@@ -227,7 +220,7 @@ def _tracemin_fiedler(L, X, normalized, tol, method):
         # element needs to modified. Changing to infinity forces a zero in the
         # corresponding element in the solution.
         i = (A.indptr[1:] - A.indptr[:-1]).argmax()
-        A[i, i] = float("inf")
+        A[i, i] = np.inf
         solver = _LUSolver(A)
     else:
         raise nx.NetworkXError(f"Unknown linear system solver: {method}")
@@ -278,18 +271,13 @@ def _get_fiedler_func(method):
 
         def find_fiedler(L, x, normalized, tol, seed):
             import scipy as sp
-            import scipy.sparse  # call as sp.sparse
-            import scipy.sparse.linalg  # call as sp.sparse.linalg
 
             L = sp.sparse.csc_array(L, dtype=float)
             n = L.shape[0]
             if normalized:
-                # TODO: rm csc_array wrapping when spdiags array becomes available
-                D = sp.sparse.csc_array(
-                    sp.sparse.spdiags(
-                        1.0 / np.sqrt(L.diagonal()), [0], n, n, format="csc"
-                    )
-                )
+                D = sp.sparse.dia_array(
+                    (1.0 / np.sqrt(L.diagonal()), 0), shape=(n, n)
+                ).tocsc()
                 L = D @ L @ D
             if method == "lanczos" or n < 10:
                 # Avoid LOBPCG when n < 10 due to
@@ -301,8 +289,7 @@ def _get_fiedler_func(method):
                 return sigma[1], X[:, 1]
             else:
                 X = np.asarray(np.atleast_2d(x).T)
-                # TODO: rm csr_array wrapping when spdiags array becomes available
-                M = sp.sparse.csr_array(sp.sparse.spdiags(1.0 / L.diagonal(), 0, n, n))
+                M = sp.sparse.dia_array((1.0 / L.diagonal(), 0), shape=(n, n)).tocsr()
                 Y = np.ones(n)
                 if normalized:
                     Y /= D.diagonal()
@@ -317,8 +304,9 @@ def _get_fiedler_func(method):
     return find_fiedler
 
 
-@np_random_state(5)
 @not_implemented_for("directed")
+@np_random_state(5)
+@nx._dispatchable(edge_attrs="weight")
 def algebraic_connectivity(
     G, weight="weight", normalized=False, tol=1e-8, method="tracemin_pcg", seed=None
 ):
@@ -404,16 +392,17 @@ def algebraic_connectivity(
 
     L = nx.laplacian_matrix(G)
     if L.shape[0] == 2:
-        return 2.0 * L[0, 0] if not normalized else 2.0
+        return 2.0 * float(L[0, 0]) if not normalized else 2.0
 
     find_fiedler = _get_fiedler_func(method)
     x = None if method != "lobpcg" else _rcm_estimate(G, G)
     sigma, fiedler = find_fiedler(L, x, normalized, tol, seed)
-    return sigma
+    return float(sigma)
 
 
-@np_random_state(5)
 @not_implemented_for("directed")
+@np_random_state(5)
+@nx._dispatchable(edge_attrs="weight")
 def fiedler_vector(
     G, weight="weight", normalized=False, tol=1e-8, method="tracemin_pcg", seed=None
 ):
@@ -510,6 +499,7 @@ def fiedler_vector(
 
 
 @np_random_state(5)
+@nx._dispatchable(edge_attrs="weight")
 def spectral_ordering(
     G, weight="weight", normalized=False, tol=1e-8, method="tracemin_pcg", seed=None
 ):
@@ -592,6 +582,7 @@ def spectral_ordering(
     return order
 
 
+@nx._dispatchable(edge_attrs="weight")
 def spectral_bisection(
     G, weight="weight", normalized=False, tol=1e-8, method="tracemin_pcg", seed=None
 ):
@@ -635,7 +626,7 @@ def spectral_bisection(
         See :ref:`Randomness<randomness>`.
 
     Returns
-    --------
+    -------
     bisection : tuple of sets
         Sets with the bisection of nodes
 
@@ -656,4 +647,4 @@ def spectral_bisection(
     nodes = np.array(list(G))
     pos_vals = v >= 0
 
-    return set(nodes[~pos_vals]), set(nodes[pos_vals])
+    return set(nodes[~pos_vals].tolist()), set(nodes[pos_vals].tolist())
