@@ -1,5 +1,5 @@
-"""Modularity matrix of graphs.
-"""
+"""Modularity matrix of graphs."""
+
 import networkx as nx
 from networkx.utils import not_implemented_for
 
@@ -8,6 +8,7 @@ __all__ = ["modularity_matrix", "directed_modularity_matrix"]
 
 @not_implemented_for("directed")
 @not_implemented_for("multigraph")
+@nx._dispatchable(edge_attrs="weight")
 def modularity_matrix(G, nodelist=None, weight=None):
     r"""Returns the modularity matrix of G.
 
@@ -39,7 +40,7 @@ def modularity_matrix(G, nodelist=None, weight=None):
 
     Returns
     -------
-    B : Numpy matrix
+    B : Numpy array
       The modularity matrix of G.
 
     Examples
@@ -71,19 +72,12 @@ def modularity_matrix(G, nodelist=None, weight=None):
     # Expected adjacency matrix
     X = np.outer(k, k) / (2 * m)
 
-    import warnings
-
-    warnings.warn(
-        "modularity_matrix will return a numpy array instead of a matrix in NetworkX 3.0.",
-        FutureWarning,
-        stacklevel=2,
-    )
-    # TODO: rm np.asmatrix for networkx 3.0
-    return np.asmatrix(A - X)
+    return A - X
 
 
 @not_implemented_for("undirected")
 @not_implemented_for("multigraph")
+@nx._dispatchable(edge_attrs="weight")
 def directed_modularity_matrix(G, nodelist=None, weight=None):
     """Returns the directed modularity matrix of G.
 
@@ -116,7 +110,7 @@ def directed_modularity_matrix(G, nodelist=None, weight=None):
 
     Returns
     -------
-    B : Numpy matrix
+    B : Numpy array
       The modularity matrix of G.
 
     Examples
@@ -169,12 +163,4 @@ def directed_modularity_matrix(G, nodelist=None, weight=None):
     # Expected adjacency matrix
     X = np.outer(k_out, k_in) / m
 
-    import warnings
-
-    warnings.warn(
-        "directed_modularity_matrix will return a numpy array instead of a matrix in NetworkX 3.0.",
-        FutureWarning,
-        stacklevel=2,
-    )
-    # TODO: rm np.asmatrix for networkx 3.0
-    return np.asmatrix(A - X)
+    return A - X
